import java.io.IOException;
import java.io.Serializable;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.Hashtable;
import java.util.Vector;


public class SimpleP2PClient extends Thread implements SimpleP2PClientListener {
	
	ServerSocket server ;
	SimpleP2PClientListener listener = this ;
	
	public SimpleP2PClient(int listenPort) throws IOException {
		server = new ServerSocket(listenPort) ;
		System.out.println("SERVER RUNNING ON PORT "+ listenPort);
		
		this.setDaemon(false) ;
		
		start() ;
	}
	
	public SimpleP2PClientListener getListener() {
		return listener;
	}
	
	public void setListener(SimpleP2PClientListener listener) {
		this.listener = listener;
	}

	public void run() {
		try {
			Socket socket = server.accept() ;
			if (socket != null) new SocketHandler(this, new SimpleP2PSocket(this, socket) ) ;
		}
		catch (Exception e) { e.printStackTrace() ;}
		
	}
	
	static private class SocketHandler extends Thread {
		SimpleP2PClient client ;
		SimpleP2PSocket socket ;
		
		public SocketHandler(SimpleP2PClient client, SimpleP2PSocket socket) throws IOException {
			this.client = client ;
			this.socket = socket ;
			
			this.setDaemon(false) ;
			
			start();
		}

		public void run() {
			
			try {
				while (socket.isConnected()) {
					SimpleP2PMsg msg = socket.readMsg() ;
					
					//System.out.println("READ> "+ msg);
					
					if (msg.getLevel() == 0) {
						try {
							client.listener.onReceiveMessage(client, msg) ;
						}
						catch (Exception e) { return ;}	
					}
					else {
						client.addResponse(msg) ;
					}
					
				}	
			}
			catch (Exception e) { return ;}
			
		}
		
	}

	public void onReceiveMessage(SimpleP2PClient client, SimpleP2PMsg msg) {}
	
	//////////////////////////////////////////////////////////////////////
	
	Hashtable<String, SimpleP2PSocket> sockets = new Hashtable<String, SimpleP2PSocket>() ;

	Vector<SimpleP2PMsg> responses = new Vector<SimpleP2PMsg>() ;
	
	protected void addResponse(SimpleP2PMsg msg) {
		synchronized (responses) {
			responses.add(msg) ;
			responses.notifyAll() ;
		}
	}
	
	protected SimpleP2PMsg getResponse(SimpleP2PMsg msg) {
		return getResponse(msg) ;
	}
	
	protected SimpleP2PMsg getResponse(long id , int level) {
		++level ;
		synchronized (responses) {
			SimpleP2PMsg msg = null ;
			
			while (msg == null) {
				for (int i = 0; i < responses.size(); i++) {
					SimpleP2PMsg resp = responses.get(i) ;
					if (resp.getId() == id && resp.getLevel() == level) {
						msg = resp ;
						responses.remove(i) ;
						break ;
					}
				}
				
				if (msg == null) {
					try { responses.wait() ;} catch (Exception e) {}
				}
			}
			
			return msg ;
		}
	}
	
	public SimpleP2PSocket getSocket(String peerHost) {
		SimpleP2PSocket socket ;
		synchronized (sockets) {
			socket = sockets.get(peerHost) ;
			if (socket == null) {
				String[] parts = peerHost.split(":") ;
				String host = parts[0] ;
				int port = 2345 ;
				
				if (parts.length > 1) {
					try {
						port = Integer.parseInt(parts[1]) ;
					}
					catch (NumberFormatException e) {}
				}
				
				try {
					socket = new SimpleP2PSocket(this, host , port) ;
					if (socket != null && socket.isConnected()) {
						sockets.put(peerHost , socket) ;
						new SocketHandler(this , socket) ;
					}
				}
				catch (Exception e) { e.printStackTrace() ;}
			}
		}
		
		return socket ;
	}
	
	public SimpleP2PMsg sendMessage(String peerHost , String title , String text , Serializable data) throws IOException {
		SimpleP2PSocket socket = getSocket(peerHost) ;
		
		SimpleP2PMsg msg = new SimpleP2PMsg(this, socket , title , text, data) ;
		
		socket.writeMsg(msg) ;
		
		return msg ;
	}
		
}
