import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;


public class SimpleP2PMsg {
	
	SimpleP2PClient client ;
	SimpleP2PSocket socket ;

	long id ;
	int level = 0 ;
	
	String title ;
	String text ;
	Serializable data ;
	
	static int idCount = -1 ;
	static public long newID() {
		int precision = 100000 ;
        long id = System.currentTimeMillis() * (precision*10);
        
        int rand = ((int)(Math.random() * precision)) * 10 ;
        
        id += rand + (++idCount) ;
        
        return id;
    }
	
	public SimpleP2PMsg(SimpleP2PClient client, SimpleP2PSocket socket, String title, String text, Serializable data) {
		this.client = client ;
		this.id = newID() ;
		this.level = 0 ;
		
		this.socket = socket;
		this.title = title;
		this.text = text;
		this.data = data;
	}
	
	public SimpleP2PMsg(SimpleP2PClient client, SimpleP2PSocket socket, byte[] block) {
		this.client = client ;
		this.socket = socket;
		
		try {
			ByteArrayInputStream bin = new ByteArrayInputStream(block) ;
			ObjectInputStream objIn = new ObjectInputStream(bin) ;
			
			this.id = objIn.readLong();
			this.level = objIn.readInt();
			this.title = (String)objIn.readObject() ;
			this.text = (String)objIn.readObject() ;
			this.data = (Serializable)objIn.readObject() ;
		}
		catch (Exception e) { e.printStackTrace() ;}
	}

	public long getId() {
		return id;
	}
	
	public int getLevel() {
		return level;
	}
	
	public byte[] getSerial() { 
		ByteArrayOutputStream bout = new ByteArrayOutputStream() ;
		try {
			ObjectOutputStream objOut = new ObjectOutputStream(bout) ;
			
			objOut.writeLong(id);
			objOut.writeInt(level);
			objOut.writeObject(title) ;
			objOut.writeObject(text) ;
			objOut.writeObject(data) ;
			
			objOut.flush() ;
			
			return bout.toByteArray() ;
		}
		catch (Exception e) { e.printStackTrace() ;}
		return null ;
	}
	
	public SimpleP2PMsg replyMessage(String title , String text, Serializable data) throws IOException {
		SimpleP2PMsg msg = new SimpleP2PMsg(client, socket , title , text, data) ;
		msg.id = this.id ;
		msg.level = this.level+1 ;
		
		socket.writeMsg(msg) ;
		
		return msg ;
	}
	
	public SimpleP2PMsg readResponse() {
		return client.getResponse(id , level) ;
	}
	
	@Override
	public String toString() {
		String str = this.getClass().getName() ;
		str += "[id: "+id +" | level: "+ level+"][ title: "+ title +" | test: "+ text +" | data: "+ data +"]" ;
		
		return str ;
	}
}
